//
// ReparentingBehavior
//

Type.registerNamespace('Web.UI.WebControls');

Web.UI.WebControls.ReparentingBehavior = function(element) {

    Web.UI.WebControls.ReparentingBehavior.initializeBase(this, [element]);

    // properties
    this._parentControlID = null;
    this._parentElement = null;
    this._oldParentElement = null;
    
    // handlers
    this._beginRequestHandler = null;
    this._endRequestHandler = null;
}

Web.UI.WebControls.ReparentingBehavior.prototype = {

    initialize : function() {
        Web.UI.WebControls.ReparentingBehavior.callBaseMethod(this, 'initialize');

        var pageRequestManager = Sys.WebForms.PageRequestManager.getInstance();
        if (pageRequestManager) {
            this._beginRequestHandler = Function.createDelegate(this, this.beginRequest);
            pageRequestManager.add_beginRequest(this._beginRequestHandler);

            this._endRequestHandler = Function.createDelegate(this, this.endRequest);
            pageRequestManager.add_endRequest(this._endRequestHandler);
        }
        
        this._parentElement = document.forms[0];
        
        this.reparent();
    },

    dispose : function() {
        this.unparent();

        var pageRequestManager = Sys.WebForms.PageRequestManager.getInstance();
        if (pageRequestManager) {
            pageRequestManager.remove_beginRequest(this.beginRequest);
            this._beginRequestHandler = null;

            pageRequestManager.remove_endRequest(this.endRequest);
            this._endRequestHandler = null;
        }

        Web.UI.WebControls.ReparentingBehavior.callBaseMethod(this, 'dispose');
    },

    beginRequest : function(sender, arg) {
        this.unparent();
    },

    endRequest : function(sender, arg) {
        this.reparent();
    },
    
    reparent : function() {
        if (this._parentElement == null)
            return;
            
        var elem = this.get_element();
        if (elem == null)
            return;
            
	    if (elem.parentElement != this._parentElement) {
	        this._oldParentElement = elem.parentElement;
		    elem.parentElement.removeChild(elem);
		    this._parentElement.appendChild(elem);
	    }
    },
    
    unparent : function() {
        if (this._oldParentElement == null)
            return;
            
        var elem = this.get_element();
        if (elem == null)
            return;
            
        this._parentElement.removeChild(elem);
        this._oldParentElement.appendChild(elem);
        this._oldParentElement = null;
    },

    //
    // Properties
    //
    get_parentControlID : function() {
        return this._parentControlID;
    },
    
    set_parentControlID : function(value) {
        this._parentControlID = value;
    }
}

Web.UI.WebControls.ReparentingBehavior.registerClass('Web.UI.WebControls.ReparentingBehavior', AjaxControlToolkit.BehaviorBase);
